/* dnl.c */
/*
   dmg's network lookup 
   output name of host, or hosts on C, B, or A class network.

   author: Damian M Gryski <dmg@idirect.com>
   finished: Dec 1/97

 */

#include <arpa/inet.h>
#include <ctype.h>
#include <getopt.h>
#include <netdb.h>
#include <netinet/in.h>
#include <stdio.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <unistd.h>

#define version 0.01
#define revision 0

#define STREQ(a, b) (strcmp((a), (b))==0)

void usage(int status);
char *getfqdn(int a, int b, int c, int d);
void getips(FILE * infile);
char *prog;

int 
main(int argc, char *argv[])
{
   int i, j;
   int anet, bnet, cnet, dnet;
   int bstart, cstart, dstart;
   int bend, cend, dend;

   FILE *infile;

   char *s;

   prog = argv[0];
   infile = NULL;

   if (argc == 1)
      usage(0);

   for (i = 1; i < argc; i++) {
      bstart = cstart = dstart = 0;
      bend = cend = dend = 255;
      if (STREQ(argv[i], "--help"))
	 usage(0);
      else if (argv[i][0] == '-') {
	 switch (argv[i][1]) {
	    case NULL:
	       infile = stdin;
	       break;
	    case 'f':
	       if (++i >= argc) {
		  fprintf(stderr, "%s: missing argument to -f\n", prog);
		  usage(1);
	       }
	       if ((infile = fopen(argv[i], "r")) == NULL) {
		  fprintf(stderr, "%s: can't open:", prog);
		  perror(argv[i]);
	       }
	       break;
	    case 'h':
	       usage(0);
	       break;
	 }
      }
      if (infile)
	 getips(infile);
      else {
	 j = sscanf(argv[i], "%d.%d.%d.%d", &anet, &bnet, &cnet, &dnet);
	 switch (j) {
	    case 4:
	       dstart = dend = dnet;
	    case 3:
	       cstart = cend = cnet;
	    case 2:
	       bstart = bend = bnet;
	    case 1:
	       break;
	    case 0:
	       usage(1);
	 }

/* here's the real work */
	 for (bnet = bstart; bnet <= bend; bnet++)
	    for (cnet = cstart; cnet <= cend; cnet++)
	       for (dnet = dstart; dnet <= dend; dnet++)
		  if ((s = getfqdn(anet, bnet, cnet, dnet)))
		     printf("%d.%d.%d.%d\t%s\n", anet, bnet, cnet, dnet, s);

      }
   }
   return 0;
}

void
usage(int status)
{
   printf("dmg network lookup %.2f%c\n", version, revision);
   printf("usage :\t%s ip[.ip[.ip[.ip]]] | -f filename | -]\n", prog);
   exit(status);
}

char *
getfqdn(int a, int b, int c, int d)
{
   char s[80];
   static struct hostent *host;
   unsigned long addr;

   sprintf(s, "%d.%d.%d.%d", a, b, c, d);
   if ((addr = inet_addr(s)) == -1)
      return NULL;
   if ((host = gethostbyaddr((char *) &addr, sizeof(long), AF_INET)) != NULL)
        return host->h_name;
   return NULL;
}

void
getips(FILE * ifile)
{
   char *t;
   int a, b, c, d, retval = 0;

   while (retval != EOF) {
      if ((retval = fscanf(ifile, "%d.%d.%d.%d\n", &a, &b, &c, &d)) == EOF)
	 break;
      if (retval != 4) {
	 while (!isdigit(retval = getc(ifile)) && (retval != EOF));
	 ungetc(retval, ifile);
	 continue;
      }
      if ((t = getfqdn(a, b, c, d)))
	 printf("%d.%d.%d.%d\t%s\n", a, b, c, d, t);
   }
   return;
}
